#include "miniengine.h"
#include "part2.h"
#include "shared.h"
#include "sprite.h"

LPD3DXMESH				pRectangle;
D3DXVECTOR4				colorTable[7];

void SAPart2Init()
{
	MiniEngine::GetFX("fx/default.fx");
	MiniEngine::GetRT("FS_A8R8G8B8", -1, -1, D3DFMT_A8R8G8B8);
	MiniEngine::GetRT("FS2_A8R8G8B8", -1, -1, D3DFMT_A8R8G8B8);
	MiniEngine::GetTexture("textures/mosaic.dds");

	D3DXCreatePolygon(MiniEngine::GetDevice(), 2.0f, 4, &pRectangle, NULL);
	pRectangle->OptimizeInplace(D3DXMESHOPT_COMPACT|D3DXMESHOPT_ATTRSORT|D3DXMESHOPT_VERTEXCACHE, NULL, NULL, NULL, NULL);

	for (int i=0;i<7;i++)
	{
		colorTable[i].x=cosf(D3DX_PI*4.2f*((float)i/7.0f))*0.1f+1.0f;
		colorTable[i].y=cosf(D3DX_PI*3.4f*((float)i/7.0f))*0.1f+1.0f;
		colorTable[i].z=cosf(D3DX_PI*1.5f*((float)i/7.0f))*0.1f+1.0f;
		colorTable[i].w=0;
	}
}

void SAPart2Terminate()
{
	SAFE_RELEASE(pRectangle);
}

void SAPart2Render(float fPhase, float fBloomStrength, float fScale, float fBias, float fIntensity, float fAdd1, float fAdd2, float fAdd3)
{
	const LPD3DXEFFECT& pDefaultFX = MiniEngine::GetFX("fx/default.fx");
	const MiniEngine::RenderTarget& actualTarget = MiniEngine::GetRT("FS_A8R8G8B8", -1, -1, D3DFMT_A8R8G8B8);
	const MiniEngine::RenderTarget& prevTarget = MiniEngine::GetRT("FS2_A8R8G8B8", -1, -1, D3DFMT_A8R8G8B8);

	BYTE iIntensity=(BYTE)(fIntensity*255.0f);
	MiniEngine::GetDevice()->SetRenderTarget(0, actualTarget.GetSurface());
	MiniEngine::GetDevice()->SetDepthStencilSurface(MiniEngine::GetDepthBuffer());
	MiniEngine::GetDevice()->Clear( 0, NULL, D3DCLEAR_TARGET, D3DCOLOR_ARGB(iIntensity,iIntensity,iIntensity,iIntensity), 1.0f, 0 );

	D3DXMATRIX mProj;
	D3DXMatrixPerspectiveFovLH(&mProj, D3DXToRadian(60.0f), (float)MiniEngine::GetBackBufferDesc().Width/(float)MiniEngine::GetBackBufferDesc().Height, 0.00001f, 10000.0f);
	SHSetWorldViewProjBias(NULL, &mSHIdentity, &mProj, &actualTarget.GetTextureBiasMatrix());

	D3DXMATRIX mScaling, mRotation, mTranslation;
	D3DXMatrixScaling(&mScaling, 10.0f, 10.0f, 10.0f);
	pDefaultFX->SetTexture("tTexture", MiniEngine::GetTexture("textures/mosaic.dds").GetTexture());

//	float fMove=fmodf(fTime*0.2f, 0.8f);
	float fMove=fmodf(fPhase, 0.8f);
	for (int i=6;i>0;i--)
	{
		float fColorLerp=fMove/0.8f;
		D3DXVECTOR4 rgb=(colorTable[i-1]*fColorLerp+colorTable[i]*(1.0f-fColorLerp))*fIntensity;
		if (i==1) rgb=rgb+D3DXVECTOR4(fAdd1, fAdd1*0.8f, fAdd1*0.8f, 0.0f);
		if (i==2) rgb=rgb+D3DXVECTOR4(fAdd2*0.8f, fAdd2, fAdd2*0.8f, 0.0f);
		if (i==3) rgb=rgb+D3DXVECTOR4(fAdd3*0.8f, fAdd3*0.8f, fAdd3, 0.0f);


		float fZ=(float)i*0.8f-fMove;
		float fAnim=(fTime*0.3f)+fZ;
		float fAlpha=fZ<0.5f ? fZ*2.0f : (fZ>1.0f) ? (5.0f-fZ)/4.0f : 1.0f;

		pSharedFX->SetVector("vUVTransform", &D3DXVECTOR4(8.0f, 8.0f, fAnim, fAnim*0.3f));
		D3DXMatrixRotationYawPitchRoll(&mRotation, D3DXToRadian(180.0f), 0, fAnim);
		D3DXMatrixTranslation(&mTranslation, 0, 0, fZ);
		D3DXMATRIX mWorld = mRotation*mScaling*mTranslation;
		SHSetWorldViewProjBias(&mWorld, NULL, NULL, NULL);

		pSharedFX->SetVector("vConstColour", &D3DXVECTOR4(rgb.x, rgb.y, rgb.z, fAlpha));
		MiniEngine::DrawMeshWithFX(pDefaultFX, "TexturedBlend", pRectangle);
	}

	const MiniEngine::RenderTarget& bloomTarget=SHBlur(1.0f, 3, D3DXVECTOR4(fBloomStrength, fBloomStrength, fBloomStrength, 1), 4.0f);

	MiniEngine::GetDevice()->SetRenderTarget(0, MiniEngine::GetBackBuffer());
	MiniEngine::GetDevice()->SetDepthStencilSurface(MiniEngine::GetDepthBuffer());
	
	pSpriteFX->SetTexture("tTexture2", actualTarget.GetTexture());
	pSpriteFX->SetTexture("tTexture3", bloomTarget.GetTexture());
	pSpriteFX->SetVector("vZoomerCompositeBias", &D3DXVECTOR4(fScale, fBias, 0.0f, 0.0f));
	SADrawSprite("ZoomerComposite", prevTarget.GetTexture(),  D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), D3DXVECTOR4(1.0f, 1.0f, 1.0f, min(1.0f, fTimeDelta*5.0f)));
	MiniEngine::GetDevice()->StretchRect(MiniEngine::GetBackBuffer(), NULL, prevTarget.GetSurface(), NULL, D3DTEXF_POINT);
}
