#include "miniengine.h"
#include "shared.h"
#include "sprite.h"
#include "bass.h"

D3DXMATRIX	mSHIdentity;
D3DXMATRIX	mSHWorld;
D3DXMATRIX	mSHView;
D3DXMATRIX	mSHProj;
D3DXMATRIX	mSHBias;
D3DXMATRIX	mSHWorldView;
D3DXMATRIX	mSHWorldViewProj;
D3DXMATRIX	mSHViewProj;
D3DXMATRIX	mSHViewProjBias;
D3DXMATRIX	mSHProjBias;
D3DXMATRIX	mSHInvView;
D3DXMATRIX	mSHInvProj;
D3DXMATRIX	mSHFrustum;
D3DXVECTOR4	vSHEyePos;

LPD3DXEFFECT	pSharedFX=NULL;

float		fTimeFirst;
float		fTime;
float		fTimePrev;
float		fTimeDelta;
float		fPattern;

HSTREAM		hMP3;

	float fSecondsStart=34.23f;
	float fSecondsPerPattern=(2.5f/115.0f)*6.0f*64.0f;

bool SHIsPlaying()
{
	return BASS_ChannelIsActive(hMP3)==BASS_ACTIVE_PLAYING;
}

void SHInit()
{
	D3DXMatrixIdentity(&mSHIdentity);
	SHResetMatrices();
	fTimeFirst=(float)timeGetTime()*0.001f;
	fTimePrev=0;
	fTimeDelta=0;
	pSharedFX=MiniEngine::GetFX("fx/shared.fx");
	pSharedFX->AddRef();

	fPattern=0;
}

void SHStart()
{
	BASS_Init(-1,44100,0,GetForegroundWindow(),NULL);
	hMP3=BASS_StreamCreateFile(false, "sound/slide2.mp3", 0, 0, BASS_STREAM_PRESCAN);
	BASS_ChannelPlay(hMP3, false);

// uncomment this to set an alternative starting point for the demo. for testing purposes.
#if 0
	float fSecondsStart=34.23f;
	float fSecondsPerPattern=(2.5f/115.0f)*6.0f*64.0f;
	BASS_ChannelSetPosition(hMP3, BASS_ChannelSeconds2Bytes(hMP3, fSecondsStart+fSecondsPerPattern*11.0f));
	BASS_ChannelSetPosition(hMP3, BASS_ChannelSeconds2Bytes(hMP3, fSecondsPerPattern*2.0f));
#endif

	SHNewFrame();
}

void SHTerminate()
{
	BASS_Free();

	SAFE_RELEASE(pSharedFX);
}

void SHNewFrame()
{
	fTimePrev=fTime;

	fTime=BASS_ChannelBytes2Seconds(hMP3, BASS_ChannelGetPosition(hMP3));
	fTimeDelta=fTime-fTimePrev;
	float fPattern2;
	if (fTime<fSecondsStart)
	{
		fPattern2=fTime/fSecondsPerPattern;
	} else {
		fPattern2=((fTime-fSecondsStart)/fSecondsPerPattern)+5;
	}

	if (fPattern2<fPattern)
	{
		float abcd=1;
	}
	fPattern=fPattern2;
}

void SHResetMatrices()
{
	D3DXMatrixIdentity(&mSHWorld);
	D3DXMatrixIdentity(&mSHView);
	D3DXMatrixIdentity(&mSHProj);
	D3DXMatrixIdentity(&mSHProjBias);
	D3DXMatrixIdentity(&mSHWorldView);
	D3DXMatrixIdentity(&mSHWorldViewProj);
	D3DXMatrixIdentity(&mSHViewProj);
	D3DXMatrixIdentity(&mSHViewProjBias);
	D3DXMatrixIdentity(&mSHProjBias);
	D3DXMatrixIdentity(&mSHInvView);
	D3DXMatrixIdentity(&mSHInvProj);
	D3DXMatrixIdentity(&mSHFrustum);
}

void SHSetWorldViewProjBias(const D3DXMATRIX* pWorld, const D3DXMATRIX* pView, const D3DXMATRIX *pProj, const D3DXMATRIX *pBias)
{
	const LPD3DXEFFECT& pSharedFX=MiniEngine::GetFX("fx/shared.fx");
	if (pWorld) { pSharedFX->SetMatrix("mWorld", pWorld); mSHWorld=*pWorld; }
	if (pView) { pSharedFX->SetMatrix("mView", pView); mSHView=*pView; }
	if (pProj) { pSharedFX->SetMatrix("mProj", pProj); mSHProj=*pProj; }
	if (pBias) { pSharedFX->SetMatrix("mBias", pBias); mSHBias=*pBias; }

	if (pView || pProj)
	{
		// viewproj
		mSHViewProj=mSHView*mSHProj;
		pSharedFX->SetMatrix("mViewProj", &mSHViewProj);

		// invview
		D3DXMatrixInverse(&mSHInvView, NULL, &mSHView);
		pSharedFX->SetMatrix("mInvView", &mSHInvView);
		vSHEyePos=D3DXVECTOR4(mSHInvView(3,0), mSHInvView(3,1), mSHInvView(3,2), 1.0f);
		pSharedFX->SetVector("vEyePos", &vSHEyePos);

		// invproj
		D3DXMatrixInverse(&mSHInvProj, NULL, &mSHProj);
		pSharedFX->SetMatrix("mInvProj", &mSHInvProj);

		// frustum
		D3DXVECTOR4 vCorner(1.0f, 1.0f, 0.0f, 1.0f);
		D3DXVECTOR4 vCornerBack;
		D3DXVec4Transform(&vCornerBack, &vCorner, &mSHInvProj);

		D3DXMatrixScaling(&mSHFrustum, vCornerBack.x, vCornerBack.y, 0.0f);
		mSHFrustum(3,2)=1.0f;
		pSharedFX->SetMatrix("mFrustum", &mSHFrustum);
	}

	if (pWorld || pView)
	{
		mSHWorldView=mSHWorld*mSHView;
		pSharedFX->SetMatrix("mWorldView", &mSHWorldView);
	}

	if (pWorld || pView || pProj)
	{
		mSHWorldViewProj=mSHWorld*mSHViewProj;
		pSharedFX->SetMatrix("mWorldViewProj", &mSHWorldViewProj);
	}

	if (pView || pProj || pBias)
	{
		mSHViewProjBias=mSHViewProj*mSHBias;
		mSHProjBias=mSHProj*mSHBias;
		pSharedFX->SetMatrix("mViewProjBias", &mSHViewProjBias);
		pSharedFX->SetMatrix("mProjBias", &mSHProjBias);
	}
}

const MiniEngine::RenderTarget& SHBlur(float fBlurSize, int iNumPasses, const D3DXVECTOR4& vBlurColour, float fBlurThreshold)
{
	const MiniEngine::RenderTarget& BlurTarget1=MiniEngine::GetRT("HS_A8R8G8B8", -2, -2, D3DFMT_A8R8G8B8);
	const MiniEngine::RenderTarget& BlurTarget2=MiniEngine::GetRT("HS2_A8R8G8B8", -2, -2, D3DFMT_A8R8G8B8);

	LPDIRECT3DSURFACE9 pTempTarget=NULL, pTempDepth=NULL;
	MiniEngine::GetDevice()->GetRenderTarget(0, &pTempTarget);
	MiniEngine::GetDevice()->GetDepthStencilSurface(&pTempDepth);

	if (fBlurThreshold==0.0f)
	{
		MiniEngine::GetDevice()->StretchRect(pTempTarget, NULL, BlurTarget1.GetSurface(), NULL, D3DTEXF_LINEAR);
	} else {
		MiniEngine::GetDevice()->StretchRect(pTempTarget, NULL, BlurTarget2.GetSurface(), NULL, D3DTEXF_LINEAR);
		MiniEngine::GetDevice()->SetDepthStencilSurface(NULL);
		MiniEngine::GetDevice()->SetRenderTarget(0, BlurTarget1.GetSurface());
		pSpriteFX->SetFloat("blurThreshold", fBlurThreshold);
		SADrawSprite("BlurExtract", BlurTarget2.GetTexture(), D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f));
	}

	D3DXVECTOR4 vRealBlurColour=vBlurColour/7.0f;
	for (int i=0;i<iNumPasses;i++)
	{
		MiniEngine::GetDevice()->SetRenderTarget(0, BlurTarget2.GetSurface());
		pSpriteFX->SetVector("blurSize", &D3DXVECTOR4(fBlurSize*float(i+1), 0.0f, 0.0f, 0.0f));
		SADrawSprite("Blur", BlurTarget1.GetTexture(), D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), vRealBlurColour);
		MiniEngine::GetDevice()->SetRenderTarget(0, BlurTarget1.GetSurface());
		pSpriteFX->SetVector("blurSize", &D3DXVECTOR4(0.0f, fBlurSize*float(i+1), 0.0f, 0.0f));
		SADrawSprite("Blur", BlurTarget2.GetTexture(), D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), vRealBlurColour);
	}

	MiniEngine::GetDevice()->SetRenderTarget(0, pTempTarget);
	MiniEngine::GetDevice()->SetDepthStencilSurface(pTempDepth);
	SAFE_RELEASE(pTempTarget);
	SAFE_RELEASE(pTempDepth);
	return BlurTarget2;
}

