#include "miniengine.h"
#include "sprite.h"
#include "shared.h"

LPD3DXEFFECT	pSpriteFX=NULL;

struct SASpriteVertex
{
	D3DXVECTOR3		m_vPosition;
	D3DXVECTOR2		m_vUV;
};

void SASpriteInit()
{
	pSpriteFX=MiniEngine::GetFX("fx/sprites.fx");
	pSpriteFX->AddRef();
}

void SASpriteTerminate()
{
	SAFE_RELEASE(pSpriteFX);
}

void SADrawSprite(LPCSTR sTechnique, const LPDIRECT3DTEXTURE9& pTexture, const D3DXVECTOR4& srcRect, const D3DXVECTOR4& dstRect, const D3DXVECTOR4 vColour, float fOptionalZ)
{
	LPDIRECT3DSURFACE9 pTempSurface;

	MiniEngine::GetDevice()->GetRenderTarget(0, &pTempSurface);
	D3DSURFACE_DESC textureDesc, targetDesc;
	if (pTexture) pTexture->GetLevelDesc(0, &textureDesc); else memset(&textureDesc, 0, sizeof(D3DSURFACE_DESC));
	pTempSurface->GetDesc(&targetDesc);

	u_short spriteIndices[6]={0,1,2,1,3,2};
	SASpriteVertex spriteVertices[4];
	spriteVertices[0].m_vPosition=D3DXVECTOR3(dstRect.x*2.0f-1.0f, dstRect.y*-2.0f+1.0f, fOptionalZ);
	spriteVertices[0].m_vUV=D3DXVECTOR2(srcRect.x, srcRect.y);
	spriteVertices[1].m_vPosition=D3DXVECTOR3(dstRect.z*2.0f-1.0f, dstRect.y*-2.0f+1.0f, fOptionalZ);
	spriteVertices[1].m_vUV=D3DXVECTOR2(srcRect.z, srcRect.y);
	spriteVertices[2].m_vPosition=D3DXVECTOR3(dstRect.x*2.0f-1.0f, dstRect.w*-2.0f+1.0f, fOptionalZ);
	spriteVertices[2].m_vUV=D3DXVECTOR2(srcRect.x, srcRect.w);
	spriteVertices[3].m_vPosition=D3DXVECTOR3(dstRect.z*2.0f-1.0f, dstRect.w*-2.0f+1.0f, fOptionalZ);
	spriteVertices[3].m_vUV=D3DXVECTOR2(srcRect.z, srcRect.w);

	pSharedFX->SetVector("vConstColour", &vColour);
	pSpriteFX->SetTexture("tTexture", pTexture);
	pSpriteFX->SetVector("vTexelSize", &D3DXVECTOR4(1.0f/(float)textureDesc.Width, 1.0f/(float)textureDesc.Height, 0.5f/(float)textureDesc.Width, 0.5f/(float)textureDesc.Height));
	pSpriteFX->SetVector("vPixelSize", &D3DXVECTOR4(1.0f/(float)targetDesc.Width, 1.0f/(float)targetDesc.Height, 0.5f/(float)targetDesc.Width, 0.5f/(float)targetDesc.Height));
	MiniEngine::GetDevice()->SetFVF(D3DFVF_XYZ|D3DFVF_TEX1);
	MiniEngine::DrawUPWithFX(pSpriteFX, sTechnique, (void*)&spriteVertices[0], sizeof(SASpriteVertex), (void*)&spriteIndices[0], 4, 2);

	SAFE_RELEASE(pTempSurface);
}
