#include "miniengine.h"
#include "testpart.h"
#include "shared.h"
#include "sprite.h"

LPD3DXMESH				pTeapot;
LPD3DXMESH				pBox;
LPD3DXMESH				pPlane;
#define NUMPOS			30
D3DXVECTOR3				vPositions[NUMPOS];
#define RANDOMSPHERERADIUS	0.5f
D3DXVECTOR4				vRandomSphere[128];
LPDIRECT3DTEXTURE9		pRandomPlanes;
LPDIRECT3DTEXTURE9		pRandomJitter;
D3DXVECTOR4				vRandomTaps[4];
#define NUMBIGYO 3000
D3DXVECTOR3				vRandomBigyo[NUMBIGYO];
D3DXVECTOR3				vRandomBigyoAxis[NUMBIGYO];
float					vRandomBigyoRandom[NUMBIGYO];
LPDIRECT3DVERTEXBUFFER9	pTriangles;

struct vertex
{
	D3DXVECTOR3	pos;
	D3DXVECTOR3	norm;
};

DWORD fvf=D3DFVF_XYZ|D3DFVF_NORMAL;

D3DXVECTOR2 GetRandomDiskPoint()
{
	D3DXVECTOR2 vPoint;
	float r=sqrtf(FRAND);
	float t=2.0f*D3DX_PI*FRAND;
	vPoint.x=r*cosf(t);
	vPoint.y=r*sinf(t);
	return vPoint;
}

D3DXVECTOR3 GetRandomSpherePoint()
{
	D3DXVECTOR3 vPoint;
#if 1
	vPoint.z = FRAND*2.0f-1.0f;
	float t = 2.0f*D3DX_PI*FRAND;
	float w = sqrtf(1.0f-vPoint.z*vPoint.z);
	vPoint.x = w*cosf(t);
	vPoint.y = w*sinf(t);
#else
	vPoint.x=FRAND*2.0f-1.0f;
	vPoint.y=FRAND*2.0f-1.0f;
	vPoint.z=FRAND*2.0f-1.0f;
	D3DXVec3Normalize(&vPoint, &vPoint);
#endif
	return vPoint;
}

void SATestInit()
{
	MiniEngine::GetFX("fx/default.fx");
	MiniEngine::GetFX("fx/water.fx");
	MiniEngine::GetRT("FS_R32F", -1, -1, D3DFMT_R32F);
	MiniEngine::GetRT("FS_D24S8", -1, -1, D3DFMT_D24S8);
	MiniEngine::GetRT("HS_A8R8G8B8", -2, -2, D3DFMT_A8R8G8B8);
	MiniEngine::GetRT("HS_D24S8", -2, -2, D3DFMT_D24S8);
	MiniEngine::GetRT("HS_R32F", -2, -2, D3DFMT_R32F);
	MiniEngine::GetRT("1024_R32F", 1024, 1024, D3DFMT_R32F);
	MiniEngine::GetRT("1024_D24S8", 1024, 1024, D3DFMT_D24S8);
	MiniEngine::GetFX("fx/default.fx");
	MiniEngine::GetTexture("textures/noisevolume2.dds");
	MiniEngine::GetTexture("textures/white.dds");

	D3DXCreateTeapot(MiniEngine::GetDevice(), &pTeapot, NULL);
	pTeapot->OptimizeInplace(D3DXMESHOPT_COMPACT|D3DXMESHOPT_ATTRSORT|D3DXMESHOPT_VERTEXCACHE, NULL, NULL, NULL, NULL);
	D3DXCreatePolygon(MiniEngine::GetDevice(), 2.0f, 4, &pPlane, NULL);
	pPlane->OptimizeInplace(D3DXMESHOPT_COMPACT|D3DXMESHOPT_ATTRSORT|D3DXMESHOPT_VERTEXCACHE, NULL, NULL, NULL, NULL);
	D3DXCreateBox(MiniEngine::GetDevice(), 1.0f, 1.0f, 1.0f, &pBox, NULL);
	pBox->OptimizeInplace(D3DXMESHOPT_COMPACT|D3DXMESHOPT_ATTRSORT|D3DXMESHOPT_VERTEXCACHE, NULL, NULL, NULL, NULL);

	MiniEngine::GetDevice()->CreateVertexBuffer(sizeof(vertex)*NUMBIGYO*3, 0, fvf, D3DPOOL_MANAGED, &pTriangles, NULL);

#if 0
// tetraeder kod
// +++
// -+-
// --+
// +--
	struct vertex
	{
		D3DXVECTOR3	pos;
		D3DXVECTOR3	norm;
	};

	vertex *vert;
	WORD *ind;
	pTeapot->LockVertexBuffer(0, (void**)&vert);
	pTeapot->LockIndexBuffer(0, (void**)&ind);

	D3DXVECTOR3 tetra[4] = { D3DXVECTOR3(+1, +1, +1),
							D3DXVECTOR3(-1, +1, -1),
							D3DXVECTOR3(-1, -1, +1),
							D3DXVECTOR3(+1, -1, -1) };

	for (int i=0;i<4;i++)
	{
		D3DXVECTOR3 v0=tetra[i];
		D3DXVECTOR3 v1=tetra[(i+1)%4];
		D3DXVECTOR3 v2=tetra[(i+2)%4];
		D3DXVECTOR3 v3=tetra[(i+3)%4];

		D3DXVECTOR3 n0 = ((v1-v0)+(v2-v0))*0.5f;
		D3DXVec3Normalize(&n0, &n0);
		D3DXVECTOR3 n1 = ((v0-v1)+(v2-v1))*0.5f;
		D3DXVec3Normalize(&n1, &n1);
		D3DXVECTOR3 v012 = v0+n0*0.3f;
		D3DXVECTOR3 v102 = v1+n0*0.3f;

		vert->pos=v0;
		vert->norm=D3DXVECTOR3(0,1,0);
		vert++;
		vert->pos=v1;
		vert->norm=D3DXVECTOR3(0,1,0);
		vert++;
		vert->pos=v012;
		vert->norm=D3DXVECTOR3(0,1,0);
		vert++;
		vert->pos=v102;
		vert->norm=D3DXVECTOR3(0,1,0);
		vert++;

		*ind++=i*4+0;
		*ind++=i*4+1;
		*ind++=i*4+2;
		*ind++=i*4+1;
		*ind++=i*4+3;
		*ind++=i*4+2;
	}
	for (int i=8;i<pTeapot->GetNumFaces();i++)
	{
		*ind++=0;
		*ind++=0;
		*ind++=0;
	}

	pTeapot->UnlockIndexBuffer();
	pTeapot->UnlockVertexBuffer();
#endif

	for (int i=0;i<NUMPOS;i++) vPositions[i]=D3DXVECTOR3(FRAND*10.0f-5.0f, FRAND*10.0f, FRAND*10.0f-5.0f);

	for (int i=0;i<128;i++)
	{
		D3DXVECTOR3 vPoint=GetRandomSpherePoint();
		float r = FRAND*RANDOMSPHERERADIUS;
		vRandomSphere[i]=D3DXVECTOR4(vPoint.x*r, vPoint.y*r, vPoint.z*r, 0.0f);
	}

	for (int i=0;i<4;i++)
	{
		D3DXVECTOR2 vPoint=GetRandomDiskPoint();
		vPoint*=8.0f/1024.0f;
		vRandomTaps[i]=D3DXVECTOR4(vPoint.x, vPoint.y, 0.0f, 0.0f);
	}

	MiniEngine::GetDevice()->CreateTexture(MiniEngine::GetBackBufferDesc().Width, MiniEngine::GetBackBufferDesc().Height, 1, 0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, &pRandomPlanes, NULL);
	MiniEngine::GetDevice()->CreateTexture(MiniEngine::GetBackBufferDesc().Width, MiniEngine::GetBackBufferDesc().Height, 1, 0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, &pRandomJitter, NULL);
	D3DLOCKED_RECT lockedRect, lockedRect2;
	pRandomPlanes->LockRect(0, &lockedRect, NULL, 0);
	pRandomJitter->LockRect(0, &lockedRect2, NULL, 0);
	for (u_int i=0;i<MiniEngine::GetBackBufferDesc().Height;i++)
	{
		D3DCOLOR *pColour=(D3DCOLOR*)((BYTE*)lockedRect.pBits+lockedRect.Pitch*i);
		D3DCOLOR *pJitter=(D3DCOLOR*)((BYTE*)lockedRect2.pBits+lockedRect2.Pitch*i);
		for (u_int j=0;j<MiniEngine::GetBackBufferDesc().Width;j++)
		{
			{
				D3DXVECTOR3 vPoint=GetRandomSpherePoint();
				BYTE x=(BYTE)(vPoint.x*127.5f+127.5f);
				BYTE y=(BYTE)(vPoint.y*127.5f+127.5f);
				BYTE z=(BYTE)(vPoint.z*127.5f+127.5f);
				*pColour++=D3DCOLOR_RGBA(x, y, z, 255);
			}
			{
				D3DXVECTOR2 vPoint=GetRandomDiskPoint();
				BYTE x=(BYTE)(vPoint.x*127.5f+127.5f);
				BYTE y=(BYTE)(vPoint.y*127.5f+127.5f);
				*pJitter++=D3DCOLOR_RGBA(x, y, 128, 128);
			}
		}
	}
	pRandomJitter->UnlockRect(0);
	pRandomPlanes->UnlockRect(0);

	for (int i=0;i<NUMBIGYO;i++)
	{
		vRandomBigyo[i]=D3DXVECTOR3(FRAND*16.0f, FRAND*7.0f, FRAND*16.0f);
		vRandomBigyoAxis[i]=GetRandomSpherePoint();
		vRandomBigyoRandom[i]=FRAND*D3DX_PI*2.0f;
	}
}

void SARenderScene(float fLocalPattern, const LPD3DXEFFECT& pFX, LPCSTR sTechnique)
{
	float ySin=0;//(sinf(fTime*4)*0.5f+0.5f)*2.0f;
	D3DXMATRIX mTranslate, mRotate, mScale;
	D3DXMATRIX mWorld;
/*	for (int i=0;i<NUMPOS;i++)
	{
		D3DXMatrixTranslation(&mWorld, vPositions[i].x, vPositions[i].y+ySin, vPositions[i].z);
		D3DXMatrixRotationY(&mRotate, fTime+(float)i);
		mWorld=mRotate*mWorld;
		SHSetWorldViewProjBias(&mWorld, NULL, NULL, NULL);
		MiniEngine::DrawMeshWithFX(pFX, sTechnique, pBox);
	}*/


	float ls=1.0f-max(0.0f, min(1.0f, fLocalPattern));
	ls=1.0f-ls*ls;
//	if (ls>0)
//	{
		float endscale=max(0.0f, min(1.0f, (fLocalPattern-3.75f)*4.0f));
		endscale=pow(endscale, 2.0f);
		D3DXMatrixTranslation(&mTranslate, 0, 3.0f, 0);
		for (int i=0;i<200;i++)
		{
			float s=sinf((float)i+fTime*0.2f)*0.5f+0.5f;
			s=s*ls;

			D3DXVECTOR3 vscale=D3DXVECTOR3(s*15.0f+endscale*10.0f, 0.2f*(ls>0 ? 1 : 0)+endscale, 0.2f*(ls>0 ? 1 : 0)+endscale);
			float vrot=vRandomBigyoRandom[i]+fTime*0.1f;
			D3DXMatrixScaling(&mScale, vscale.x, vscale.y, vscale.z);
			D3DXMatrixRotationAxis(&mRotate, &vRandomBigyoAxis[i], vrot);

			mWorld=mScale*mRotate*mTranslate;
			SHSetWorldViewProjBias(&mWorld, NULL, NULL, NULL);
			MiniEngine::DrawMeshWithFX(pFX, sTechnique, pBox);
		}
//	} else {
		if (sTechnique!="ShadowMap")
		{
			SHSetWorldViewProjBias(&mSHIdentity, NULL, NULL, NULL);
			D3DXMatrixScaling(&mScale, 0.05f, 0.02f, 0.02f);
			vertex *vertices;
			pTriangles->Lock(0, sizeof(vertex)*NUMBIGYO*3, (void**)&vertices, 0);
			for (int i=0;i<NUMBIGYO;i++)
			{
				D3DXMatrixRotationAxis(&mRotate, &vRandomBigyoAxis[i], fTime+(float)i);
				mWorld=mScale*mRotate;
				mWorld(3,0)=8.0f-fmodf(vRandomBigyo[i].x+fTime*0.1f,16.0f);
				mWorld(3,1)=6.7f-fmodf(vRandomBigyo[i].y+fTime*0.5f,7.0f);
				mWorld(3,2)=8.0f-fmodf(vRandomBigyo[i].z+fTime*0.2f,16.0f);

				D3DXVec3TransformCoord(&vertices->pos, &D3DXVECTOR3(-1.0f, -1.0f, 0.0f), &mWorld);
				D3DXVec3TransformNormal(&vertices->norm, &D3DXVECTOR3(0.0f, 1.0f, 0.0f), &mWorld);
				vertices++;

				D3DXVec3TransformCoord(&vertices->pos, &D3DXVECTOR3(+1.0f, -1.0f, 0.0f), &mWorld);
				D3DXVec3TransformNormal(&vertices->norm, &D3DXVECTOR3(0.0f, 1.0f, 0.0f), &mWorld);
				vertices++;

				D3DXVec3TransformCoord(&vertices->pos, &D3DXVECTOR3(+1.0f, +1.0f, 0.0f), &mWorld);
				D3DXVec3TransformNormal(&vertices->norm, &D3DXVECTOR3(0.0f, 1.0f, 0.0f), &mWorld);
				vertices++;

//				MiniEngine::DrawMeshWithFX(pFX, sTechnique, pPlane);
			}
			pTriangles->Unlock();
			DWORD trs;
			MiniEngine::GetDevice()->GetRenderState(D3DRS_CULLMODE, &trs);
			MiniEngine::GetDevice()->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
			MiniEngine::GetDevice()->SetStreamSource(0, pTriangles, 0, sizeof(vertex));
			MiniEngine::GetDevice()->SetFVF(fvf);
			MiniEngine::DrawWithFX(pFX, sTechnique, NUMBIGYO);
			MiniEngine::GetDevice()->SetRenderState(D3DRS_CULLMODE, trs);
		}
//	}
}

void SACalculateViewProj(float fLocalPattern, D3DXMATRIX& mView, D3DXMATRIX& mProj)
{
	float ftime=0;//fTime*0.3f;
	D3DXVECTOR3 mLook(cosf(ftime)*10, 2.0f, sinf(ftime)*10);
	D3DXVECTOR3 mAt(cosf(ftime)*1, 0.0f, cosf(ftime)*1);
	D3DXVECTOR3 mUp(0.0f, 1.0f, 0.0f);

	mLook=D3DXVECTOR3(0, 0, -fLocalPattern*5.0f);
	mAt=D3DXVECTOR3(0, 1, 0);
	D3DXMatrixLookAtLH(&mView, &mLook, &mAt, &mUp);
	D3DXMatrixPerspectiveFovLH(&mProj, D3DXToRadian(60.0f), (float)MiniEngine::GetBackBufferDesc().Width/(float)MiniEngine::GetBackBufferDesc().Height, 0.1f, 1000.0f);
}

void SATestRender2(float fLocalPattern)
{
	const MiniEngine::RenderTarget& ZColour = MiniEngine::GetRT("FS_R32F", -1, -1, D3DFMT_R32F);
	const MiniEngine::RenderTarget& ZDepth = MiniEngine::GetRT("FS_D24S8", -1, -1, D3DFMT_D24S8);
	const LPD3DXEFFECT& pDefaultFX = MiniEngine::GetFX("fx/default.fx");

	D3DXMATRIX mView, mProj;
	SACalculateViewProj(fLocalPattern, mView, mProj);

	MiniEngine::GetDevice()->SetRenderTarget(0, ZColour.GetSurface());
	MiniEngine::GetDevice()->SetDepthStencilSurface(ZDepth.GetSurface());
	MiniEngine::GetDevice()->Clear(0, NULL, D3DCLEAR_ZBUFFER|D3DCLEAR_STENCIL, D3DCOLOR_ARGB(0, 0, 0, 0), 1.0f, 0);
	SHSetWorldViewProjBias(NULL, &mView, &mProj, &ZColour.GetTextureBiasMatrix());

//	SARenderScene(0, pDefaultFX, "Depth");

	D3DXMATRIX mWorld, mScale, mRotate, mTranslate;
	float fs=fLocalPattern*0.2f;
	D3DXMatrixScaling(&mScale, 50.0f*fs, 1.0f*fs, 1.0f*fs);
	for (int i=0;i<50;i++)
	{
		D3DXVECTOR3 rt=vRandomBigyoAxis[i]*5.0f;
		D3DXMatrixTranslation(&mTranslate, rt.x, rt.y, rt.z);
		D3DXMatrixRotationAxis(&mRotate, &vRandomBigyoAxis[i], vRandomBigyoRandom[i]+fTime*0.1f);
		mWorld=mScale*mRotate*mTranslate;
		SHSetWorldViewProjBias(&mWorld, NULL, NULL, NULL);
		MiniEngine::DrawMeshWithFX(pDefaultFX, "Depth", pBox);
	}

	SADrawSprite("ClearAODepth", NULL, D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), D3DXVECTOR4(65536.0f, 1.0f, 1.0f, 1.0f), 0.999999f);

	MiniEngine::GetDevice()->SetRenderTarget(0, MiniEngine::GetBackBuffer());
	MiniEngine::GetDevice()->SetDepthStencilSurface(MiniEngine::GetDepthBuffer());

	u_int uiNumAOPasses=6;
	u_int uiNumAOSamples=5;

	pSpriteFX->SetTexture("tRandomPlanes", pRandomPlanes);
	pSpriteFX->SetFloat("AOcontrib", 1.0f/((float)uiNumAOSamples*(float)uiNumAOPasses));
	for (u_int i=0;i<uiNumAOPasses;i++)
	{
		pSpriteFX->SetVectorArray("vRandomSphere", &vRandomSphere[i*uiNumAOSamples], uiNumAOSamples);
		SADrawSprite(i ? "AmbientOcclusionAdd" : "AmbientOcclusion", ZColour.GetTexture(), D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f));
	}

//	SADrawSprite("Sprite", ZColour.GetTexture(), D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), D3DXVECTOR4(0.0f, 0.0f, 0.25f, 0.25f), D3DXVECTOR4(0.05f, 0.0f, 0.0f, 1.0f));

	if (fLocalPattern>3.75f)
	{
		float fAlpha=(fLocalPattern-3.75f)*2.0f;
		fAlpha=max(0.0f, fAlpha*fAlpha);
		SADrawSprite("SpriteBlend", (LPDIRECT3DTEXTURE9)MiniEngine::GetTexture("textures/white.dds").GetTexture(), D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), D3DXVECTOR4(0.0f, 0.0f, 0.0f, fAlpha));
	}

}

void SARenderWaterScene(float fLocalPattern, const LPD3DXEFFECT& pFX, LPCSTR sTechnique,
						const LPD3DXEFFECT& pWaterFX, LPCSTR sWaterTechnique)
{
	if (sTechnique) SARenderScene(fLocalPattern, pFX, sTechnique);
	if (sWaterTechnique)
	{
		// water plane (in normal view)
		D3DXMATRIX mScaling, mRotation;
		D3DXMatrixRotationX(&mRotation, D3DXToRadian(270.0f));
		D3DXMatrixScaling(&mScaling, 50.0f, 50.0f, 50.0f);
		D3DXMATRIX mWorld = mScaling*mRotation;

		SHSetWorldViewProjBias(&mWorld, NULL, NULL, NULL);
		MiniEngine::DrawMeshWithFX(pWaterFX, sWaterTechnique, pPlane);
	}
}

void SATestRender(float fLocalPattern, const D3DXVECTOR3& lookFrom, const D3DXVECTOR3& lookAt, const D3DXVECTOR3& lookUp, float fFade)
{
	const MiniEngine::RenderTarget& MirrorColour = MiniEngine::GetRT("HS_A8R8G8B8", -2, -2, D3DFMT_A8R8G8B8);
	const MiniEngine::RenderTarget& MirrorDepth = MiniEngine::GetRT("HS_D24S8", -2, -2, D3DFMT_D24S8);
	const MiniEngine::RenderTarget& MirrorZColour = MiniEngine::GetRT("HS_R32F", -2, -2, D3DFMT_R32F);
	const MiniEngine::RenderTarget& ShadowColour = MiniEngine::GetRT("1024_R32F", 1024, 1024, D3DFMT_R32F);
	const MiniEngine::RenderTarget& ShadowDepth = MiniEngine::GetRT("1024_D24S8", 1024, 1024, D3DFMT_D24S8);
	const MiniEngine::RenderTarget& ZColour = MiniEngine::GetRT("FS_R32F", -1, -1, D3DFMT_R32F);
	const MiniEngine::RenderTarget& ZDepth = MiniEngine::GetRT("FS_D24S8", -1, -1, D3DFMT_D24S8);
	const LPD3DXEFFECT& pDefaultFX = MiniEngine::GetFX("fx/default.fx");

	// render shadowmap
	D3DXMATRIX mShadowView, mShadowProj;

	MiniEngine::GetDevice()->SetRenderTarget(0, ShadowColour.GetSurface());
	MiniEngine::GetDevice()->SetDepthStencilSurface(ShadowDepth.GetSurface());
	MiniEngine::GetDevice()->Clear(0, NULL, D3DCLEAR_ZBUFFER|D3DCLEAR_STENCIL, D3DCOLOR_ARGB(0, 0, 0, 0), 1.0f, 0);
	D3DXVECTOR3 mLook(18.0f, 18.0f, -18.0f);
	D3DXVECTOR3 mAt(0.0f, 0.0f, 0.0f);
	D3DXVECTOR3 mUp(0.0f, 1.0f, 0.0f);

	D3DXMatrixLookAtLH(&mShadowView, &mLook, &mAt, &mUp);
	D3DXMatrixPerspectiveFovLH(&mShadowProj, D3DXToRadian(40.0f), 1.5f, 0.1f, 1000.0f);
	SHSetWorldViewProjBias(NULL, &mShadowView, &mShadowProj, &ShadowColour.GetTextureBiasMatrix());

	pSharedFX->SetMatrix("mShadowViewProjBias", &mSHViewProjBias);
	pSharedFX->SetTexture("tShadowTexture", ShadowColour.GetTexture());
	pSharedFX->SetTexture("tShadowJitterTexture", pRandomJitter);
	pSharedFX->SetVectorArray("vRandomTaps", &vRandomTaps[0], 4);
	pSharedFX->SetVector("vPointLightPos", &D3DXVECTOR4(mLook.x, mLook.y, mLook.z, 1.0f));
	pSharedFX->SetVector("vPointLightColour", &D3DXVECTOR4(1.0f*fFade, 1.0f*fFade, 1.0f*fFade, 1.0f));
	pSharedFX->SetVector("vAmbientColour", &D3DXVECTOR4(0.15f*fFade, 0.15f*fFade, 0.15f*fFade, 1.0f));

	mShadowView(3,2)+=0.15f; // shadow bias
	SHSetWorldViewProjBias(NULL, &mShadowView, NULL, NULL);
	SARenderWaterScene(fLocalPattern, pDefaultFX, "ShadowMap", NULL, NULL);
	SADrawSprite("ClearAODepth", NULL, D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), D3DXVECTOR4(1000.0f, 1.0f, 1.0f, 1.0f), 0.999999f);

	// set view/proj
	D3DXMATRIX mView, mProj;
//	SACalculateViewProj(mView, mProj);

	D3DXMatrixLookAtLH(&mView, &lookFrom, &lookAt, &lookUp);
	D3DXMatrixPerspectiveFovLH(&mProj, D3DXToRadian(60.0f), (float)MiniEngine::GetBackBufferDesc().Width/(float)MiniEngine::GetBackBufferDesc().Height, 0.1f, 50.0f);


	// calculate the mirrored view and an oblique fustrum matrix
	D3DXMATRIX mReflectedView, mObliqueProj;
	MiniEngine::Reflection(mView, mProj, D3DXPLANE(0.0f, 1.0f, 0.0f, 0.0f), mReflectedView, mObliqueProj);

	// depth in mirror
	MiniEngine::GetDevice()->SetRenderTarget(0, MirrorZColour.GetSurface());
	MiniEngine::GetDevice()->SetDepthStencilSurface(MirrorDepth.GetSurface());
	MiniEngine::GetDevice()->Clear(0, NULL, D3DCLEAR_ZBUFFER|D3DCLEAR_STENCIL, D3DCOLOR_ARGB(0, 0, 0, 0), 1.0f, 0);
	SHSetWorldViewProjBias(NULL, &mReflectedView, &mObliqueProj, &ZColour.GetTextureBiasMatrix());
	MiniEngine::GetDevice()->SetRenderState(D3DRS_CULLMODE, D3DCULL_CW);
	SARenderWaterScene(fLocalPattern, pDefaultFX, "Depth", pDefaultFX, "Depth");
	MiniEngine::GetDevice()->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);
	SADrawSprite("ClearAODepth", NULL, D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), D3DXVECTOR4(50.0f, 1.0f, 1.0f, 1.0f), 0.999999f);

	// render the scene with faces flipped
	MiniEngine::GetDevice()->SetRenderTarget(0, MirrorColour.GetSurface());
	MiniEngine::GetDevice()->SetDepthStencilSurface(MirrorDepth.GetSurface());
	MiniEngine::GetDevice()->Clear(0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER|D3DCLEAR_STENCIL, D3DCOLOR_ARGB(0, 0, 0, 0), 1.0f, 0);
	SHSetWorldViewProjBias(NULL, &mReflectedView, &mObliqueProj, &MirrorColour.GetTextureBiasMatrix());
	MiniEngine::GetDevice()->SetRenderState(D3DRS_CULLMODE, D3DCULL_CW);
	SARenderWaterScene(fLocalPattern, pDefaultFX, "Shadowed", NULL, NULL);
	MiniEngine::GetDevice()->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);

	// fog sprite in mirror
	pSpriteFX->SetTexture("tVolume", MiniEngine::GetTexture("textures/noisevolume2.dds").GetTexture());
//	SADrawSprite("VolumeFog", MirrorZColour.GetTexture(), D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), D3DXVECTOR4(0.5f, 0.5f, 0.6f, 1.0f));
	SADrawSprite("VolumeFog", MirrorZColour.GetTexture(), D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), D3DXVECTOR4(fFade, fFade, fFade, 1));

	// depth in normal view
	MiniEngine::GetDevice()->SetRenderTarget(0, ZColour.GetSurface());
	MiniEngine::GetDevice()->SetDepthStencilSurface(ZDepth.GetSurface());
	MiniEngine::GetDevice()->Clear(0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER|D3DCLEAR_STENCIL, D3DCOLOR_ARGB(0, 0, 0, 0), 1.0f, 0);
	SHSetWorldViewProjBias(NULL, &mView, &mProj, &ZColour.GetTextureBiasMatrix());
	SARenderWaterScene(fLocalPattern, pDefaultFX, "Depth", pDefaultFX, "Depth");
	SADrawSprite("ClearAODepth", NULL, D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), D3DXVECTOR4(50.0f, 1.0f, 1.0f, 1.0f), 0.999999f);

	// scene in normal view
	MiniEngine::GetDevice()->SetRenderTarget(0, MiniEngine::GetBackBuffer());
	MiniEngine::GetDevice()->SetDepthStencilSurface(MiniEngine::GetDepthBuffer());
	MiniEngine::GetDevice()->Clear( 0, NULL, D3DCLEAR_ZBUFFER|D3DCLEAR_STENCIL, D3DCOLOR_ARGB(255,0,0,0), 1.0f, 0 );

	SHSetWorldViewProjBias(NULL, &mView, &mProj, &MiniEngine::GetBackBufferTextureBias());
	const LPD3DXEFFECT& pWaterFX=MiniEngine::GetFX("fx/water.fx");
	pWaterFX->SetTexture("tMirrorTexture", MirrorColour.GetTexture());
	SARenderWaterScene(fLocalPattern, pDefaultFX, "Shadowed", pWaterFX, "Water");

	// fog sprite
	pSpriteFX->SetTexture("tVolume", MiniEngine::GetTexture("textures/noisevolume2.dds").GetTexture());
	SADrawSprite("VolumeFog", ZColour.GetTexture(), D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), D3DXVECTOR4(fFade, fFade, fFade, 1));

	// debug
//	SADrawSprite("Sprite", MirrorColour.GetTexture(), D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), D3DXVECTOR4(0.0f, 0.0f, 0.25f, 0.25f));
//	SADrawSprite("Sprite", ShadowColour.GetTexture(), D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), D3DXVECTOR4(0.0f, 0.0f, 0.25f, 0.25f));
//	SADrawSprite("Sprite", MirrorZColour.GetTexture(), D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), D3DXVECTOR4(0.0f, 0.0f, 0.25f, 0.25f), D3DXVECTOR4(0.1f, 0, 0, 0));
//	SADrawSprite("Sprite", MirrorColour.GetTexture(), D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f), D3DXVECTOR4(0.0f, 0.0f, 0.25f, 0.25f));
}

void SATestTerminate()
{
	SAFE_RELEASE(pRandomJitter);
	SAFE_RELEASE(pRandomPlanes);
	SAFE_RELEASE(pTeapot);
	SAFE_RELEASE(pPlane);
	SAFE_RELEASE(pBox);
	SAFE_RELEASE(pTriangles);
}

