#ifndef _MINIENGINE_H_
#define _MINIENGINE_H_

// uncomment this line for more detailed d3d debug info
//#define D3D_DEBUG_INFO
#include <d3d9.h>
#include <d3dx9.h>

#define SAFE_ACTION(x, y) { if ((x)) (x)->y; }
#define SAFE_RELEASE(x) { if ((x)) { (x)->Release(); (x)=NULL; }; }

namespace MiniEngine
{
	// structures
	class Texture
	{
	public:
		const LPDIRECT3DBASETEXTURE9& GetTexture() const { return m_pTexture; }
		const D3DXVECTOR4& GetDimensions() const { return m_vDimensions; }

		void Create(LPCSTR sFileName);
		void Reset();
		Texture();
		~Texture();

	protected:
		LPDIRECT3DBASETEXTURE9	m_pTexture;
		D3DXVECTOR4				m_vDimensions;
	};

	class RenderTarget
	{
	public:
		RenderTarget();
		~RenderTarget();

		void Reset();
		void Create();
		void Create(const D3DSURFACE_DESC& surfaceDesc, u_int uiCloneWidthDivider, u_int uiCloneHeightDivider);

		// accessors
		const LPDIRECT3DTEXTURE9& GetTexture() const { return m_pTexture; }
		const LPDIRECT3DSURFACE9& GetSurface() const { return m_pSurface; }
		const D3DSURFACE_DESC& GetDesc() const { return m_surfaceDesc; }
		const D3DXMATRIX& GetTextureBiasMatrix() const { return m_mTextureBias; }

	private:
		LPDIRECT3DTEXTURE9			m_pTexture;
		LPDIRECT3DSURFACE9			m_pSurface;
		D3DSURFACE_DESC				m_surfaceDesc;
		u_int						m_uiCloneWidthDivider;
		u_int						m_uiCloneHeightDivider;
		D3DXMATRIX					m_mTextureBias;
	};

	// init / terminate
	bool Init(HWND hWnd);
	bool Terminate();
	bool BeginDraw();
	bool EndDraw();

	// get functions
	const LPDIRECT3D9& GetD3D();
	const LPDIRECT3DDEVICE9& GetDevice();
	const LPDIRECT3DSURFACE9& GetBackBuffer();
	const LPDIRECT3DSURFACE9& GetDepthBuffer();
	const D3DSURFACE_DESC& GetBackBufferDesc();
	const D3DXMATRIX& GetBackBufferTextureBias();

	// cached resources
	const LPD3DXEFFECT&			GetFX(LPCSTR sFXName);
	const Texture&				GetTexture(LPCSTR sTextureName);
	const RenderTarget&			GetRT(LPCSTR sRTName, int iWidth, int iHeight, D3DFORMAT surfaceFormat);

	// draw commands
	void DrawWithFX(const LPD3DXEFFECT& pFX, D3DXHANDLE hTechnique, u_int uiNumFaces);
	void DrawIndexedWithFX(const LPD3DXEFFECT& pFX, D3DXHANDLE hTechnique, u_int uiNumVertices, u_int uiNumFaces);
	void DrawUPWithFX(const LPD3DXEFFECT& pFX, D3DXHANDLE hTechnique, void* pVB, u_int uiStride, void *pIB, u_int uiNumVertices, u_int uiNumFaces);
	void DrawMeshWithFX(const LPD3DXEFFECT& pFX, D3DXHANDLE hTechnique, const LPD3DXMESH& pMesh);

	// misc. math stuff
	void MatrixOblique(D3DXMATRIX* pOutProjection, const D3DXPLANE* pPlane, const D3DXMATRIX* pProjection);
	void Reflection(const D3DXMATRIX& mView, const D3DXMATRIX& mProj, const D3DXPLANE& Plane, D3DXMATRIX& mNewView, D3DXMATRIX& mNewProj);
}

#endif
